part of 'exercise_plan_bloc.dart';

@immutable
abstract class ExercisePlanState extends Equatable {
  const ExercisePlanState();

  @override
  List<Object> get props => [];
}

// Display the last saved exercise plan
class ExercisePlanInitial extends ExercisePlanState {
  const ExercisePlanInitial();
}

// Loading screen
class ExercisePlanLoading extends ExercisePlanState {
  const ExercisePlanLoading();
}

// updated screen
class ExercisePlanReady extends ExercisePlanState {
  const ExercisePlanReady();
}

// error splash screen
class ExercisePlanError extends ExercisePlanState {
  final String message;
  const ExercisePlanError({required this.message});

  @override
  List<Object> get props => [message];
}
